/***************************************************************************
 *            ifaces.c
 *
 *  Mon Jun 27 04:56:42 2005
 *  Copyright  2005  Jaime Penalba Estebanez
 *  jpenalbae@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <pcap.h>
#include <errno.h>
#include <netinet/if_ether.h>
#include <netinet/ether.h> 
#include <time.h>
#include <libnet.h>
#include "screen.h"
#include "ifaces.h"

/* tcpdump header (ether.h) defines ETHER_HDRLEN)  */
#ifndef ETHER_HDRLEN 
#define ETHER_HDRLEN 14
#endif

#ifndef ETH_ALEN
#define ETH_ALEN 6
#endif

#ifndef IP_ALEN
#define IP_ALEN 4
#endif


// Shitty globals
libnet_t *libnet;
char *ourmac;
static u_char smac[ETH_ALEN];
struct p_header *temp_header;


void setip(char *disp, char *addr, char *netm)
{
	
	int d;
	struct ifreq ifr;
	struct in_addr inaddr, innet;
		
	d = socket (PF_INET,SOCK_STREAM,IPPROTO_TCP);
	sprintf(ifr.ifr_name, "%s", disp);
  
	/* rellena las estructuras y hace la llamada*/
	inet_aton (addr, &inaddr);
	inet_aton (netm, &innet);
	ifr.ifr_addr.sa_family=PF_INET;
	
	memcpy (ifr.ifr_addr.sa_data + 2, (char *)&inaddr, 4);
	if ( (ioctl (d, SIOCSIFADDR, &ifr)) != 0 )
		perror("SIOCSIFADDR");
	
	memcpy (ifr.ifr_addr.sa_data + 2, (char *)&innet, 4);
	if ( (ioctl (d, SIOCSIFNETMASK, &ifr)) != 0 )
		perror("SIOCSIFNETMASK");
	
	close(d);

}


/* Start Sniffing on given iface */
void StartSniffer(char *disp)
{
	//printf("Sniffing at: %s\n", disp);
	
	char errbuf[PCAP_ERRBUF_SIZE];
	pcap_t *descr;
	u_char *args = NULL;
	
	descr = pcap_open_live(disp,BUFSIZ,1,-1,errbuf);
	if(descr == NULL) { printf("pcap_open_live(): %s\n", errbuf); exit(1); }
	pcap_loop(descr, 0, ProccessPacket, args);

}


/* Handle Headers and IP data */
/* from the recived pcap_loop packet */
void ProccessPacket(u_char *args, struct pcap_pkthdr* pkthdr,const u_char*
        packet)
{
	
	u_int16_t type = handle_ethernet(args,pkthdr,packet);
	
	if(type == ETHERTYPE_IP)
	{	
		//printf("Type: IP\n\n");
		//handle_IP(args,pkthdr,packet);
		
	} else if(type == ETHERTYPE_ARP)
	{
		//printf("Type: ARP\n\n");
		handle_ARP(pkthdr,packet);
		
		print_screen();

		
	} else if(type == ETHERTYPE_REVARP)
	{
		printf("Type: RARP\n\n");
		print_screen();
		
 	} else
	{
		//printf("Type: Unknown\n\n");
	}
	
}



/* Handle Ethernet Header from Packet */
u_int16_t handle_ethernet
        (u_char *args, struct pcap_pkthdr *pkthdr, const u_char *packet)
{

	u_int caplen = pkthdr->caplen;
	u_int length = pkthdr->len;
	struct ether_header *eptr;  /* net/ethernet.h */
	u_short ether_type;
	struct p_header *new_header;

	if (caplen < ETHER_HDRLEN)
	{
		fprintf(stdout,"Packet length less than ethernet header length\n");
		return -1;
	}

	/* lets start with the ether header... */
	eptr = (struct ether_header *) packet;
	ether_type = ntohs(eptr->ether_type);
	char *from = ether_ntoa((struct ether_addr*)eptr->ether_shost);

	
	if(ether_type != ETHERTYPE_ARP)
	{
		return ether_type;
		
	}
	
	if (strcmp(ourmac, from ) == 0)
	{
		return 0x069;
	}
	
	new_header = (struct p_header *) malloc (sizeof(struct p_header));
	new_header->smac = (char *) malloc (sizeof(char) * strlen(from));
	new_header->dmac = (char *) malloc (sizeof(char) * 18);
	new_header->length = (unsigned int)length;
	
	sprintf(new_header->smac, "%s", from);
	sprintf(new_header->dmac, "%s", 
				ether_ntoa((struct ether_addr*)eptr->ether_dhost));
	
	temp_header = new_header;
	
	return ether_type;
}


/* Prints Info about the arp packet */
void handle_ARP(struct pcap_pkthdr *pkthdr, const u_char *packet)
{
	struct arphdr *harp;
	struct ether_arp *earp;
	//u_char ipsrc[17];
	//u_char ipdst[17];


	harp = (struct arphdr *) (packet + ETH_HLEN);
	earp = (struct ether_arp *) (packet + ETH_HLEN);


	if ( ntohs(harp->ar_op) == ARPOP_REQUEST )
	{
		//printf("\tARP: Request\n");
		//char *type = "Request";
	}
	else if ( ntohs(harp->ar_op) == ARPOP_REPLY )
	{
		struct arp_rep_l *new_arprep_l;
		new_arprep_l = (struct arp_rep_l *) malloc (sizeof(struct arp_rep_l));
		new_arprep_l->sip = (char *) malloc (sizeof(char) * 16);
		new_arprep_l->dip = (char *) malloc (sizeof(char) * 16);


		new_arprep_l->header = temp_header;
		
		sprintf(new_arprep_l->sip, "%d.%d.%d.%d",
			earp->arp_spa[0], earp->arp_spa[1],
			earp->arp_spa[2], earp->arp_spa[3]);

		sprintf(new_arprep_l->dip, "%d.%d.%d.%d",
			earp->arp_tpa[0], earp->arp_tpa[1],
			earp->arp_tpa[2], earp->arp_tpa[3]);
		
		new_arprep_l->count = 1;
		new_arprep_l->next = NULL;
		
		arprep_add(new_arprep_l);
	}
	
	/* antiguo interface
	snprintf(ipsrc, sizeof(ipsrc) - 1, "%d.%d.%d.%d",
	    earp->arp_spa[0], earp->arp_spa[1],
	    earp->arp_spa[2], earp->arp_spa[3]);
	
	snprintf(ipdst, sizeof(ipdst) - 1, "%d.%d.%d.%d",
	    earp->arp_tpa[0], earp->arp_tpa[1],
	    earp->arp_tpa[2], earp->arp_tpa[3]);
	*/
}

void lnetInit(char *disp)
{
	
	libnet = NULL;
	char *temporal;
   char error[LIBNET_ERRBUF_SIZE];
	
	libnet = libnet_init(LIBNET_LINK, disp, error);
	
	if (ourmac == NULL)
	{
		// get hwaddr
		struct libnet_ether_addr *mymac;
		mymac = libnet_get_hwaddr(libnet);
		memcpy(smac, mymac, ETH_ALEN);
		printf("Copiando mac\n");
		temporal = ether_ntoa((struct ether_addr*)smac);
		ourmac = (char *) malloc ((sizeof(char)) * strlen(temporal));
		sprintf(ourmac, "%s", temporal);
	}

}


/* Forge Arp Packet, using libnet */
void ForgeArp(char *dest_ip, char *disp)
{
	static libnet_ptag_t arp=0, eth=0;
	
	//static u_char dmac[ETH_ALEN] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
	static u_char dmac[ETH_ALEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

	static u_char sip[IP_ALEN];
	static u_char dip[IP_ALEN];
	
	// get dst ip address
   u_int32_t otherip;
   otherip = libnet_name2addr4(libnet, dest_ip, LIBNET_RESOLVE);
   memcpy(dip, (char*)&otherip, IP_ALEN);
 
   // get ipaddr
   u_int32_t myip;
   myip = libnet_get_ipaddr4(libnet);
   memcpy(sip, (char*)&myip, IP_ALEN);
	
	
	arp = libnet_build_arp(
      ARPHRD_ETHER,
      ETHERTYPE_IP,
      ETH_ALEN, IP_ALEN,
      ARPOP_REQUEST,
      smac, sip,
      dmac, dip,
      NULL, 0,
      libnet,
      arp);
 
   eth = libnet_build_ethernet(
      dmac, smac,
      ETHERTYPE_ARP,
      NULL, 0,
      libnet,
      eth);
		
   libnet_write(libnet);
}


void lnetDestroy()
{
	libnet_destroy(libnet);
}
